/*****************************************************
Library for DS18B20 temperature sensor. Based on
"Using DS18B20 digital temperature sensor on AVR
microcontrollers" - by Gerard Marull Paretas

*****************************************************/

// DS18B20 Thermometer Connections
#define THERM_PORT PORTC
#define THERM_DDR DDRC
#define THERM_PIN PINC
#define THERM_DQ PC3
// Utils
#define THERM_INPUT_MODE() THERM_DDR&=~(1<<THERM_DQ)
#define THERM_OUTPUT_MODE() THERM_DDR|=(1<<THERM_DQ)
#define THERM_LOW() THERM_PORT&=~(1<<THERM_DQ)
#define THERM_HIGH() THERM_PORT|=(1<<THERM_DQ)
// DS18B20 Function Commands
#define THERM_CMD_CONVERTTEMP 0x44
#define THERM_CMD_RSCRATCHPAD 0xbe
#define THERM_CMD_WSCRATCHPAD 0x4e
#define THERM_CMD_CPYSCRATCHPAD 0x48
#define THERM_CMD_RECEEPROM 0xb8
#define THERM_CMD_RPWRSUPPLY 0xb4
#define THERM_CMD_SEARCHROM 0xf0
#define THERM_CMD_READROM 0x33
#define THERM_CMD_MATCHROM 0x55
#define THERM_CMD_SKIPROM 0xcc
#define THERM_CMD_ALARMSEARCH 0xec

#define THERM_DECIMAL_STEPS_12BIT 625 //.0625

// Write a bit to the DS18B20
void therm_write_bit(uint8_t bit)
{
	//Pull line low for 1uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	//therm_delay(us(1));
	_delay_us(1);
	
	//If we want to write 1, release the line (if not will keep low)
	if(bit) THERM_INPUT_MODE();
	
	//Wait for 60uS and release the line
	//therm_delay(us(60));
	_delay_us(60);
	THERM_INPUT_MODE();
}

// Read a bit from the DS18B20
uint8_t therm_read_bit(void)
{
	uint8_t bit=0;

	//Pull line low for 1uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	//therm_delay(us(1));
	_delay_us(1);

	//Release line and wait for 14uS
	THERM_INPUT_MODE();
	//therm_delay(us(14));
	_delay_us(14);
	
	//Read line value
	if(THERM_PIN&(1<<THERM_DQ)) bit=1;
	
	//Wait for 45uS to end and return read value
	//therm_delay(us(45));
	_delay_us(45);
	return bit;
}

// Reads a whole byte from the DS18B20
uint8_t therm_read_byte(void)
{
	uint8_t i=8, n=0;
	
	while(i--){
		//Shift one position right and store read value
		n>>=1;
		n|=(therm_read_bit()<<7);
	}
	
	return n;
}

// Writes a whole byte to the DS18B20
void therm_write_byte(uint8_t byte)
{
	uint8_t i=8;
	
	while(i--){
	//Write actual bit and shift one position right to make the next bit ready
	therm_write_bit(byte&1);
	byte>>=1;
	}
}

// Reset the DS18B20
uint8_t therm_reset(void)
{
	uint8_t i;
	
	//Pull line low and wait for 480uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	//therm_delay(us(480));
	_delay_us(480);
	
	//Release line and wait for 60uS
	THERM_INPUT_MODE();
	//therm_delay(us(60));
	_delay_us(60);
	
	//Store line value and wait until the completion of 480uS period
	i=(THERM_PIN & (1<<THERM_DQ));
	//therm_delay(us(420));
	_delay_us(420);
	
	//Return the value read from the presence pulse (0=OK, 1=WRONG)
	return i;
}

// Write the configuration register
void therm_config(uint8_t Th, uint8_t Tl, uint8_t config)
{
	//Reset, skip ROM and send command to write Scratchpad
	therm_reset();
	therm_write_byte(THERM_CMD_SKIPROM);
	therm_write_byte(THERM_CMD_WSCRATCHPAD);
	
	// Write Alarm bytes, then configuration register
	therm_write_byte(Th);
	therm_write_byte(Tl);
	therm_write_byte(config);
	
	therm_reset();

}