/*****************************************************
Firmware for ATmega8 ROV controller. Inputs for reading 
temperature using a DS18B20, remaining battery level using
analog input, and outputs for controlling LEDs and motors.
Surface communication via USART module, hardware interface
is RS-485.

Data Types:

int8_t - Signed Char 
uint16_t - Unsigned Int 
uint32_t - Unsigned Long 
int64_t - Signed Long Long

(signed/unsigned) char - 1 byte 
(signed/unsigned) short - 2 bytes 
(signed/unsigned) int - 2 bytes 
(signed/unsigned) long - 4 bytes 
(signed/unsigned) long long - 8 bytes 
float - 4 bytes (floating point) 
double - alias to float


Project : ROV Control Board
AVR Clock frequency: 8,00 MHz

Controls
LED out PD5
Thruster Altitude PBx and X
Thruster Right PBX and X
Thruster Right PBX and X

DS18B20 Temperature Sensor PC3
Battery Level Monitor PC0
Leak Detector PC1
Light PWM on PB1
*****************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include "DS18B20.c"

// USART definitions
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)

// Global Variables
volatile uint8_t sensor_data[4];

// Reads the temperature from the DS18B20
void therm_read_temperature(void)
{
	cli();
	//Reset, skip ROM and start temperature conversion
	therm_reset();
	therm_write_byte(THERM_CMD_SKIPROM);
	therm_write_byte(THERM_CMD_CONVERTTEMP);
	sei();

	//Wait until conversion is complete
	while(!therm_read_bit());
	
	cli();
	//Reset, skip ROM and send command to read Scratchpad
	therm_reset();
	therm_write_byte(THERM_CMD_SKIPROM);
	therm_write_byte(THERM_CMD_RSCRATCHPAD);
	
	//Read Scratchpad (only 2 first bytes)
	sensor_data[0]=therm_read_byte();
	sensor_data[1]=therm_read_byte();
	therm_reset();
	sei();
	
}

// Send a byte over USART
void USART_SendByte(uint8_t data)
{
// Wait if a byte is being transmitted
while((UCSRA&(1<<UDRE)) == 0);

// Transmit data
UDR = data;
}

void ADC_start_conversion(void)
{
	//Start conversion with Interrupt after conversion is complete
	ADCSRA |= (1<<ADSC);
}

void send_sensors(void)
{
uint8_t i;

// Set to transmitt mode
PORTD |= (1<<PD2);

// Wait for the PC to catch up!
_delay_ms(10);

// Send start byte
USART_SendByte(0xAA);

// Send sensor_data array over USART
for (i=0; i<4; i++){
USART_SendByte(sensor_data[i]);
}

// Wait if a byte is being transmitted
while((UCSRA&(1<<UDRE)) == 0);
	
// Set to receive mode
PORTD &=~(1<<PD2);
}

// Gets the status of every onboard sensor, and store in an array
void refresh_sensors(void)
{
// Get temperature
therm_read_temperature();

// Get the battery voltage level
ADC_start_conversion();

// Get the Leak Sensor status
if (PINC & (1<<PC1)){
	sensor_data[3] = 0x00;
}else{
	sensor_data[3] = 0xFF;
}

}

// USART Received Data Complete interrupt service routine
ISR(USART_RXC_vect)
{
	uint8_t temp;
	static uint8_t bytesig;
	
// Store value from USART Buffer
	temp = UDR;
	
	if (bytesig == 0xFF){	
	// Set PWM duty
		OCR1A = temp * 4;			// (1023 / 255 = 4)
		bytesig = 0;
	
	}else if (temp ==0xDD){
	// Next byte is output byte
		bytesig = 0xDD;
	
	}else if (temp == 0xFF){
	// Next byte is PWM duty cycle
		bytesig = 0xFF;
		
	}else if (temp == 0xAA){
	// Time to send sensor status
		send_sensors();
		
	}else if (((temp & 128) == 0) & (bytesig == 0xDD)){
	// Must be an output byte
		bytesig = 0;
		PORTB = (temp & 0b00111100);
		PORTD = ((temp & 0b00000011)<<6);
		// Check LED bit
		if (temp & 0b01000000){
			// Start PWM
			// Timer1 Initialization for LED PWM.
			// Fast Mode, Inverted, Prescaler 1:1, 10-bit resolution
			// (TOP value is 0x03FF = 1023) giving 7,8kHz PWM frequency
			// High frequency reduces camera flicker, add a low-pass filter to eliminate
			TCCR1A = (1<<COM1A1)|(0<<COM1A0)|(1<<WGM11)|(1<<WGM10);
			TCCR1B = (0<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10);
		}else{
			// Stop PWM
			TCCR1A = 0;
			TCCR1B = 0;
			PORTB = PORTB & 0b00111100;
		}
	}
}

// Initialize USART Module
void USART_Init(void)
{

	// Enable receiver and transmitter, with interrupts
	UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
	
	// Load lower 8-bits of the baud rate value into the low byte of the UBRR register 
	UBRRL = BAUD_PRESCALE;
	// Load upper 8-bits of the baud rate value into the high byte of the UBRR register 
	UBRRH = (BAUD_PRESCALE >> 8);
	
	//Set data frame format: asynchronous mode,no parity, 1 stop bit, 8 bit size
	UCSRC = (1<<URSEL)|(0<<UMSEL)|(0<<UPM1)|(0<<UPM0)|(0<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);

}

// Initialize Analog-to-Digital Converter Module
void ADC_init(void)
{
	// AREF selected, Left-adjusted for 8-bit precision, ADC0 selected
	ADMUX|=(0<<REFS1)|(0<<REFS0)|(1<<ADLAR)|(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(0<<MUX0);

	//set prescaler to 1:64 and enable ADC complete interrupt
	ADCSRA|=(1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0);
	
}

// Interrupt routine run when conversion is complete
ISR(ADC_vect)
{
// Put 8 MSBs of conversion result in array
sensor_data[2] = ADCH;
}

// Main Program Loop
void main(void)
{

// PORTB Initialization, LEDs
PORTB = 0;
DDRB = (1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<PB2)|(1<<PB1);

// PORTC Initialization, set as input
PORTC = 0;
DDRC = 0x00;
// PORTD Initialization
PORTD = 0;
DDRD = (1<<PD7)|(1<<PD6)|(1<<PD2);

// Set to receive mode
PORTD &=~(1<<PD2);

// Initialize USART
USART_Init();

// High temp: 60C, low temp: -5C, Resolution: 11-bits for 375ms processing time
therm_config(60, (128 | 5), 0b01011111);

// Initialize Analog to Digital Converter
ADC_init();

// Enable global interrupts
sei();

while(1){
	refresh_sensors();
	};

}